package main

type GuildMember struct {
	accountId string
	level     int // 1: 会长, 2: 副会长, 3: 精英, 4 成员
}

type Guild struct {
	autoId         int64  // 公会自增id
	guildId        int64  // 公会id
	name           string // 公会名称
	leaderId       string // 公会leader
	avatar         int32  // 头像
	notice         string // 公告
	joinCond       int32  // 公会加入条件
	joinCondValue  int32  // 公会加入条件值
	totalStars     int32  // 总星星数量
	totalKills     int32  // 单局总击杀数
	chickenDinners int32  // 单局第一名数
	maxMembers     int32  // 公会最大成员数 default 30
	members        []*GuildMember
	pendingReqs    map[string]int32 // pendingAccountId -> status 0,1,2,3 pending, accept, reject, leave
}

// GuildLog 公会日志
type GuildLog struct {
	guildId   int64
	accountId string
	logType   int32
	content   string
}

// PendingReq 待审核请求
type PendingReq struct {
	accountId string
	status    int32 // 0 pending, 1 ok, 2 reject, 3 disband
}

func (g *Guild) GetGuildId() int64 {
	return g.guildId
}

// findMemberIndex 根据 AccountId 查找成员在 Members 切片中的索引
func (g *Guild) findMemberIndex(accountId string) int {
	for i, member := range g.members {
		if member.accountId == accountId {
			return i
		}
	}
	return -1
}

// GetMember 根据 AccountId 获取成员信息
func (g *Guild) GetMember(accountId string) *GuildMember {
	index := g.findMemberIndex(accountId)
	if index == -1 {
		return nil
	}
	return g.members[index]
}

// IsMember 是否是公会成员
func (g *Guild) IsMember(accountId string) bool {
	return g.findMemberIndex(accountId) >= 0
}

// IsFull 成员是否已满
func (g *Guild) IsFull() bool {
	return int32(len(g.members)) >= g.maxMembers
}

// AddMember 添加成员
func (g *Guild) AddMember(member *GuildMember) {
	if !g.IsFull() {
		g.members = append(g.members, member)
	}
}

// RemoveMember 移除成员
func (g *Guild) RemoveMember(accountId string) {
	if accountId == g.leaderId {
		return
	}
	index := g.findMemberIndex(accountId)
	if index == -1 {
		return
	}
	copy(g.members[index:], g.members[index+1:])
	g.members[len(g.members)-1] = nil
	g.members = g.members[:len(g.members)-1]
}

func (g *Guild) SetNotice(notice *string) {
	g.notice = *notice
}

func (g *Guild) GetNotice() string {
	return g.notice
}

// AddPendingReq 添加等待审核成员
func (g *Guild) AddPendingReq(p *PendingReq) {
	g.pendingReqs[p.accountId] = p.status
}

func (g *Guild) GetPendingReqStatus(accountId string) int32 {
	if pendingReqStatus, exists := g.pendingReqs[accountId]; exists {
		return pendingReqStatus
	}
	return -1
}

// RemovePendingReq 移除等待审核成员
func (g *Guild) RemovePendingReq(accountId string) {
	delete(g.pendingReqs, accountId)
}

func (g *Guild) IsInReq(accountId string) bool {
	return g.pendingReqs[accountId] == PendingReqIsJoinGuildStatusJoined
}
