package main

import (
	"cs"
	"f5"
	"github.com/golang/protobuf/proto"
	"q5"
)

type Player struct {
	cs.MsgHandlerImpl
	socket                 f5.WspCliConn
	accountId              string
	sessionId              string
	chatChannel            int
	privateTargetAccountId string // 私聊对象 accountId
	worldChannelLastId     uint64 // 世界聊天 last id
	guildChannelLastId     uint64 // 公会聊天 last id
	privateChatLastId      uint64 // 私聊 last id
}

func (p *Player) GetAccountId() string {
	return p.accountId
}

// CMSearchUser 搜索用户
func (p *Player) CMSearchUser(hdr *f5.MsgHdr, msg *cs.CMSearchUser) {
	rspMsg := new(cs.SMSearchUser)
	listUsers := friendMgr.searchUsers(msg.GetSearchKeyword())
	for _, u := range listUsers {
		friend := &cs.MFUser{
			AccountId: &u.accountId,
			Username:  &u.username,
		}
		rspMsg.Users = append(rspMsg.Users, friend)
	}
	wspListener.sendProxyMsg(hdr.Conn, hdr.SocketHandle, rspMsg)
}

// CMSearchUserByAccountId 搜索指定用户
func (p *Player) CMSearchUserByAccountId(hdr *f5.MsgHdr, msg *cs.CMSearchUserByAccountId) {
	rspMsg := new(cs.SMSearchUserByAccountId)
	user := friendMgr.searchByAccountId(msg.GetAccountId())
	if user != nil {
		rspMsg.AccountId = &user.accountId
		rspMsg.Username = &user.username
	}
	wspListener.sendProxyMsg(hdr.Conn, hdr.SocketHandle, rspMsg)
}

// CMAddFriendRequest 添加好友请求
func (p *Player) CMAddFriendRequest(hdr *f5.MsgHdr, msg *cs.CMAddFriendRequest) {
	user1Id := p.accountId
	user2Id := msg.GetTargetAccountId()
	friendMgr.addFriendRequest(user1Id, user2Id,
		func(errCode int32, errMsg string) {
			rspMsg := new(cs.SMAddFriendRequest)
			if errCode != 0 {
				rspMsg.ErrCode = &errCode
				rspMsg.ErrMsg = &errMsg
			} else {
			}
			wspListener.sendProxyMsg(hdr.Conn, hdr.SocketHandle, rspMsg)
		})
}

// CMAcceptFriendRequest 接受好友请求
func (p *Player) CMAcceptFriendRequest(hdr *f5.MsgHdr, msg *cs.CMAcceptFriendRequest) {
	user1Id := p.accountId
	user2Id := msg.GetTargetAccountId()
	friendMgr.acceptFriendRequest(user1Id, user2Id,
		func(errCode int32, errMsg string) {
			rspMsg := new(cs.SMAcceptFriendRequest)
			if errCode != 0 {
				rspMsg.ErrCode = &errCode
				rspMsg.ErrMsg = &errMsg
			} else {
			}
			wspListener.sendProxyMsg(hdr.Conn, hdr.SocketHandle, rspMsg)
		})
}

// CMRejectFriendRequest 拒绝好友请求
func (p *Player) CMRejectFriendRequest(hdr *f5.MsgHdr, msg *cs.CMRejectFriendRequest) {
	user1Id := p.accountId
	user2Id := msg.GetTargetAccountId()
	friendMgr.rejectFriendRequest(user1Id, user2Id,
		func(errCode int32, errMsg string) {
			rspMsg := new(cs.SMRejectFriendRequest)
			if errCode != 0 {
				rspMsg.ErrCode = &errCode
				rspMsg.ErrMsg = &errMsg
			} else {
			}
			wspListener.sendProxyMsg(hdr.Conn, hdr.SocketHandle, rspMsg)
		})
}

// CMListPendingFriendRequest 等待验证的好友请求
func (p *Player) CMListPendingFriendRequest(hdr *f5.MsgHdr, msg *cs.CMListPendingFriendRequest) {
	accountId := p.accountId
	user := friendMgr.getUser(accountId)

	rspMsg := &cs.SMListPendingFriendRequest{}
	rspMsg.AccountIds = user.GetAllFriendRequestAccountIds()
	wspListener.sendProxyMsg(hdr.Conn, hdr.SocketHandle, rspMsg)
}

// CMListFriend 我的好友列表
func (p *Player) CMListFriend(hdr *f5.MsgHdr, msg *cs.CMListFriend) {
	rspMsg := &cs.SMListFriend{}
	accountId := p.accountId
	for _, friendship := range friendMgr.friendships[accountId] {
		if friendship.user1.accountId != accountId {
			friend := &cs.MFUser{
				AccountId: &friendship.user1.accountId,
				Username:  &friendship.user1.username,
			}
			rspMsg.Users = append(rspMsg.Users, friend)
		} else {
			friend := &cs.MFUser{
				AccountId: &friendship.user2.accountId,
				Username:  &friendship.user2.username,
			}
			rspMsg.Users = append(rspMsg.Users, friend)
		}
	}
	f5.GetSysLog().Info("CMListFriend friends count:%d\n", len(rspMsg.Users))
	wspListener.sendProxyMsg(hdr.Conn, hdr.SocketHandle, rspMsg)
}

// CMDeleteFriendShip 删除好友
func (p *Player) CMDeleteFriendShip(hdr *f5.MsgHdr, msg *cs.CMDeleteFriendShip) {
	user1Id := p.accountId
	user2Id := msg.GetTargetAccountId()
	friendMgr.deleteFriendShip(user1Id, user2Id,
		func(errCode int32, errMsg string) {
			rspMsg := new(cs.SMDeleteFriendShip)
			if errCode != 0 {
				rspMsg.ErrCode = &errCode
				rspMsg.ErrMsg = &errMsg
			} else {
			}
			wspListener.sendProxyMsg(hdr.Conn, hdr.SocketHandle, rspMsg)
		})
}

// CMAddBlacklist 加入黑名单
func (p *Player) CMAddBlacklist(hdr *f5.MsgHdr, msg *cs.CMAddBlacklist) {
	user1Id := p.accountId
	user2Id := msg.GetTargetAccountId()
	friendMgr.addBlacklist(user1Id, user2Id,
		func(errCode int32, errMsg string) {
			rspMsg := new(cs.SMAddBlacklist)
			if errCode != 0 {
				rspMsg.ErrCode = &errCode
				rspMsg.ErrMsg = &errMsg
			} else {
			}
			wspListener.sendProxyMsg(hdr.Conn, hdr.SocketHandle, rspMsg)
		})
}

// CMRemoveBlacklist 移除黑名单
func (p *Player) CMRemoveBlacklist(hdr *f5.MsgHdr, msg *cs.CMRemoveBlacklist) {
	user1Id := p.accountId
	user2Id := msg.GetTargetAccountId()
	friendMgr.removeBlacklist(user1Id, user2Id,
		func(errCode int32, errMsg string) {
			rspMsg := new(cs.SMRemoveBlacklist)
			if errCode != 0 {
				rspMsg.ErrCode = &errCode
				rspMsg.ErrMsg = &errMsg
			} else {
			}
			wspListener.sendProxyMsg(hdr.Conn, hdr.SocketHandle, rspMsg)
		})
}

// CMFriendInfo 我的好友信息
func (p *Player) CMFriendInfo(hdr *f5.MsgHdr, msg *cs.CMFriendInfo) {
	rspMsg := new(cs.SMFriendInfo)
	accountId := p.accountId
	user := friendMgr.getUser(accountId)

	// friendships
	friends := friendMgr.listFriends(accountId)
	for _, friend := range friends {
		rspMsg.FriendshipsAccountIds = append(rspMsg.FriendshipsAccountIds, friend.accountId)
	}

	accountIds := user.GetAllFriendRequestAccountIds()
	rspMsg.PendingReqsAccountIds = accountIds

	// blacklist
	var blacklistAccountIds []string
	for bAccountId := range user.friendBlackList {
		blacklistAccountIds = append(blacklistAccountIds, bAccountId)
	}
	rspMsg.BlacklistAccountIds = blacklistAccountIds

	wspListener.sendProxyMsg(hdr.Conn, hdr.SocketHandle, rspMsg)
}

// CMSendChatMsg 发送聊天消息
func (p *Player) CMSendChatMsg(hdr *f5.MsgHdr, msg *cs.CMSendChatMsg) {
	chatChannel := *msg.ChatChannel
	if !IsValidChatChannel(chatChannel) {
		return
	}
	switch chatChannel {
	case kCCWorld:
		chatMgr.ProcWorldChat(p, msg)
	case kCCPrivate:
		chatMgr.ProcPrivateChat(p, msg)
	case kCCGuild:
		chatMgr.ProcGuildChat(p, msg)
	default:
		return
	}
}

func (p *Player) SyncPrivateChatRedPoint() {
	//rspMsg := &cs.SMDeleteFriendShip{}
	msg := &cs.SMUpdatePrivateChatRedPointNotify{}
	chatMgr.FillSMUpdatePrivateChatRedPointNotify(p, msg)
}

// CMReadMsgAndOpenChatNotify 读取聊天消息列表并且开启聊天通知
func (p *Player) CMReadMsgAndOpenChatNotify(hdr *f5.MsgHdr, msg *cs.CMReadMsgAndOpenChatNotify) {
	chatChannel := *msg.CurrChannel
	if !IsValidChatChannel(chatChannel) {
		return
	}
	if chatChannel == kCCPrivate {
		p.SyncPrivateChatRedPoint()
	}
}

// CMSetCurrPrivateChatTarget 设置当前私聊目标
func (p *Player) CMSetCurrPrivateChatTarget(hdr *f5.MsgHdr, msg *cs.CMSetCurrPrivateChatTarget) {
	p.privateTargetAccountId = msg.GetTargetAccountId()
	chatMgr.SyncPrivateChatMsg(p)
}

func (p *Player) MarkNewMsg() {
	rspMsg := &cs.SMUpdateChatRedPointNotify{}
	chatMgr.FillSMUpdateChatRedPointNotify(p, rspMsg)
	p.SendMsg(rspMsg)
}

func (p *Player) SendMsg(rspMsg proto.Message) {
	wspListener.sendProxyMsg(p.socket.Conn, p.socket.SocketHandle, rspMsg)
}

func IsValidChatChannel(chatChannel int32) bool {
	return chatChannel > kCCBegin && chatChannel < kCCEnd
}

// CMGuildInfo 我的公会信息
func (p *Player) CMGuildInfo(hdr *f5.MsgHdr, msg *cs.CMGuildInfo) {
	rspMsg := new(cs.SMGuildInfo)
	guild := guildMgr.GetGuildByAccountId(p.accountId)
	if guild == nil {
		rspMsg.Guild = nil
		rspMsg.RandomGuilds = p.FillGuild(guildMgr.RandomGuilds())
		wspListener.sendProxyMsg(hdr.Conn, hdr.SocketHandle, rspMsg)
		return
	}

	membersCount := q5.ToInt32(len(guild.members))
	maxMembers := q5.ToInt32(guild.maxMembers)

	rspGuild := &cs.MFGuild{
		AutoId:       &guild.autoId,
		GuildId:      &guild.guildId,
		Name:         &guild.name,
		LeaderId:     &guild.leaderId,
		MembersCount: &membersCount,
		MaxMembers:   &maxMembers,
		Notice:       &guild.notice,
	}
	rspMsg.Guild = rspGuild
	rspMsg.RandomGuilds = nil
	wspListener.sendProxyMsg(hdr.Conn, hdr.SocketHandle, rspMsg)
}

// CMCreateGuild 创建公会
func (p *Player) CMCreateGuild(hdr *f5.MsgHdr, msg *cs.CMCreateGuild) {
	guildMgr.CreateGuild(*msg.Name, p.accountId,
		func(errCode int32, errMsg string, guildId int64) {
			rspMsg := new(cs.SMCreateGuild)
			if errCode != 0 {
				rspMsg.ErrCode = &errCode
				rspMsg.ErrMsg = &errMsg
			} else {
				rspMsg.GuildId = &guildId
			}
			wspListener.sendProxyMsg(hdr.Conn, hdr.SocketHandle, rspMsg)
		})
}

// CMApplyToGuild 申请者加入公会
func (p *Player) CMApplyToGuild(hdr *f5.MsgHdr, msg *cs.CMApplyToGuild) {
	guildMgr.ApplyToGuild(
		msg.GetGuildId(), p.accountId,
		func(errCode int32, errMsg string) {
			rspMsg := new(cs.SMApplyToGuild)
			if errCode != 0 {
				rspMsg.ErrCode = &errCode
				rspMsg.ErrMsg = &errMsg
			}
			wspListener.sendProxyMsg(hdr.Conn, hdr.SocketHandle, rspMsg)
		},
	)
}

// CMApplyList 请求申请者列表
func (p *Player) CMApplyList(hdr *f5.MsgHdr, msg *cs.CMApplyList) {
	rspMsg := new(cs.SMApplyList)
	guild := guildMgr.GetGuildByAccountId(p.accountId)
	if guild != nil {
		for accountId, status := range guild.pendingReqs {
			applyInfo := &cs.MFApplyInfo{
				AccountId: &accountId,
				Status:    &status,
			}
			rspMsg.ApplyInfo = append(rspMsg.ApplyInfo, applyInfo)
		}
	}

	wspListener.sendProxyMsg(hdr.Conn, hdr.SocketHandle, rspMsg)
}

// CMApprove 同意申请者加入公会
func (p *Player) CMApprove(hdr *f5.MsgHdr, msg *cs.CMApprove) {
	guildMgr.Approve(
		p.accountId, msg.GetApplicantAccountId(),
		func(errCode int32, errMsg string) {
			rspMsg := new(cs.SMApprove)
			if errCode != 0 {
				rspMsg.ErrCode = &errCode
				rspMsg.ErrMsg = &errMsg
			}
			wspListener.sendProxyMsg(hdr.Conn, hdr.SocketHandle, rspMsg)
		},
	)
}

// CMReject 拒绝申请者加入公会
func (p *Player) CMReject(hdr *f5.MsgHdr, msg *cs.CMReject) {
	guildMgr.Reject(
		p.accountId, msg.GetApplicantAccountId(),
		func(errCode int32, errMsg string) {
			rspMsg := new(cs.SMReject)
			if errCode != 0 {
				rspMsg.ErrCode = &errCode
				rspMsg.ErrMsg = &errMsg
			}
			wspListener.sendProxyMsg(hdr.Conn, hdr.SocketHandle, rspMsg)
		},
	)
}

// CMLeaveGuild 离开公会
func (p *Player) CMLeaveGuild(hdr *f5.MsgHdr, msg *cs.CMLeaveGuild) {
	guildMgr.LeaveGuild(
		p.accountId,
		func(errCode int32, errMsg string) {
			rspMsg := new(cs.SMLeaveGuild)
			if errCode != 0 {
				rspMsg.ErrCode = &errCode
				rspMsg.ErrMsg = &errMsg
			}
			wspListener.sendProxyMsg(hdr.Conn, hdr.SocketHandle, rspMsg)
		},
	)
}

// CMDismissMember 开除成员
func (p *Player) CMDismissMember(hdr *f5.MsgHdr, msg *cs.CMDismissMember) {
	guildMgr.DismissMember(
		p.accountId,
		msg.GetDismissAccountId(),
		func(errCode int32, errMsg string) {
			rspMsg := new(cs.SMDismissMember)
			if errCode != 0 {
				rspMsg.ErrCode = &errCode
				rspMsg.ErrMsg = &errMsg
			}
			wspListener.sendProxyMsg(hdr.Conn, hdr.SocketHandle, rspMsg)
		},
	)
}

// CMPromoteMember 提升成员为干部
func (p *Player) CMPromoteMember(hdr *f5.MsgHdr, msg *cs.CMPromoteMember) {
	guildMgr.PromoteMember(
		p.accountId,
		msg.GetPromoteAccountId(),
		func(errCode int32, errMsg string) {
			rspMsg := new(cs.SMPromoteMember)
			if errCode != 0 {
				rspMsg.ErrCode = &errCode
				rspMsg.ErrMsg = &errMsg
			}
			wspListener.sendProxyMsg(hdr.Conn, hdr.SocketHandle, rspMsg)
		})
}

// CMDemoteMember 解除成员干部身份
func (p *Player) CMDemoteMember(hdr *f5.MsgHdr, msg *cs.CMDemoteMember) {
	guildMgr.DemoteMember(
		p.accountId,
		msg.GetDemoteAccountId(),
		func(errCode int32, errMsg string) {
			rspMsg := new(cs.SMDemoteMember)
			if errCode != 0 {
				rspMsg.ErrCode = &errCode
				rspMsg.ErrMsg = &errMsg
			}
			wspListener.sendProxyMsg(hdr.Conn, hdr.SocketHandle, rspMsg)
		})
}

// CMDisband 解散公会
func (p *Player) CMDisband(hdr *f5.MsgHdr, msg *cs.CMDisband) {
	guildMgr.Disband(
		p.accountId,
		func(errCode int32, errMsg string) {
			rspMsg := new(cs.SMDisband)
			if errCode != 0 {
				rspMsg.ErrCode = &errCode
				rspMsg.ErrMsg = &errMsg
			}
			wspListener.sendProxyMsg(hdr.Conn, hdr.SocketHandle, rspMsg)
		})
}

// CMSetNotice 设置公告
func (p *Player) CMSetNotice(hdr *f5.MsgHdr, msg *cs.CMNotice) {
	guildMgr.SetNotice(
		p.accountId,
		msg.Content,
		func(errCode int32, errMsg string) {
			rspMsg := new(cs.SMNotice)
			if errCode != 0 {
				rspMsg.ErrCode = &errCode
				rspMsg.ErrMsg = &errMsg
			}
			wspListener.sendProxyMsg(hdr.Conn, hdr.SocketHandle, rspMsg)
		})
}

// CMSearchGuilds 搜索公会
func (p *Player) CMSearchGuilds(hdr *f5.MsgHdr, msg *cs.CMSearchGuilds) {
	rspMsg := new(cs.SMSearchGuilds)
	rspMsg.Guilds = p.FillGuild(guildMgr.SearchGuilds(msg.GetKeyword()))
	f5.GetSysLog().Info("CMSearchGuilds keyword[%s], result:%d \n", msg.GetKeyword(), len(rspMsg.Guilds))
	wspListener.sendProxyMsg(hdr.Conn, hdr.SocketHandle, rspMsg)
}

// CMGuildLogs 请求公会日志
func (p *Player) CMGuildLogs(hdr *f5.MsgHdr, msg *cs.CMGuildLogs) {
	rspMsg := new(cs.SMGuildLogs)
	guild := guildMgr.GetGuildByAccountId(p.accountId)
	if guild == nil {
		wspListener.sendProxyMsg(hdr.Conn, hdr.SocketHandle, rspMsg)
		return
	}
	for _, g := range guildMgr.GetGuildLogs(guild.GetGuildId()) {
		guildLog := &cs.MFGuildLog{
			GuildId:   &g.guildId,
			AccountId: &g.accountId,
			LogType:   &g.logType,
			Content:   &g.content,
		}
		rspMsg.GuildLogs = append(rspMsg.GuildLogs, guildLog)
	}
	wspListener.sendProxyMsg(hdr.Conn, hdr.SocketHandle, rspMsg)
}

// FillGuild 填充公共列表
func (p *Player) FillGuild(guilds []*Guild) []*cs.MFGuild {
	var resGuilds []*cs.MFGuild
	for _, g := range guilds {
		membersCount := q5.ToInt32(len(g.members))
		maxMembers := q5.ToInt32(g.maxMembers)
		guild := &cs.MFGuild{
			AutoId:       &g.autoId,
			GuildId:      &g.guildId,
			Name:         &g.name,
			LeaderId:     &g.leaderId,
			MembersCount: &membersCount,
			MaxMembers:   &maxMembers,
			Notice:       &g.notice,
		}
		resGuilds = append(resGuilds, guild)
	}
	return resGuilds
}

func (p *Player) IncrPrivateChatLastId() uint64 {
	p.privateChatLastId++
	return p.privateChatLastId
}
